from clint.textui import progress
import preprocess, gensim
import os, time, csv, requests, codecs

lib_path = os.path.dirname(os.path.abspath(__file__))
server_path = "http://103.253.146.183:8001"
corpus_path = "{}/corpus".format(lib_path)
data_path = "{}/data".format(lib_path)
dictionary_path = "{}/data".format(lib_path)
mmcorpus_path = "{}/data".format(lib_path)

dictionary_name = "review.dict"
mmcorpus_name = "review.mm"
file_name = "review.csv"

dictionary_full_path = "{}/{}".format(dictionary_path, dictionary_name)
mmcorpus_full_path = "{}/{}".format(mmcorpus_path, mmcorpus_name)
local_full_path = "{}/{}".format(data_path,file_name)
remote_full_path = "{}/{}".format(server_path, file_name)

global data
data = []

def check_directory():
    print("Preparing directory")
    if not os.path.exists(data_path): os.makedirs(data_path)
    if not os.path.exists(corpus_path): os.makedirs(corpus_path)
    print("Directory prepared!")

def download():
    check_directory()

    print("Connecting to remote server")
    r = requests.get(remote_full_path, stream=True)
    print("Connected!")
    print("Downloading review.csv")
    with open(local_full_path, 'wb') as f:
        total_length = int(r.headers.get('content-length'))
        for chunk in progress.bar(r.iter_content(chunk_size=1024), expected_size=(total_length/1024) + 1):
            if chunk:
                f.write(chunk)
                f.flush()
            #end if
        #end for
    #end with
    print("Download complete")
#end def

def load():
    global data
    print("Checking file review.csv")
    if not os.path.isfile(local_full_path):
        print("Cannot find review.csv")
        download()
    #end if
    print("Found! review.csv")

    loading_time = time.time()
    print("Loading data...")
    with open("{}/data/{}".format(lib_path,file_name), encoding="utf-8") as f:
        reader = csv.reader(f)
        data = [row for row in reader]
        del data[0]
    print("Finished in {}s".format(time.time() - loading_time))
    return data
#end def

'''Return a list of reviews.'''
def raw():
    loading_time = time.time()
    print("Loading data...")
    if len(data) == 0: load()
    print("Finished in {}s".format(time.time() - loading_time))
    return data
#end def

'''This function will make several kind of processed data such as .dict and .mm'''
def process(raw_data=None):
    check_directory()
    raw_data = raw_data if raw_data is not None else raw()
    for idx, data in enumerate(raw_data):
        print("Creating review_{}.txt".format(idx))
        with open("{}/{}".format(corpus_path,"review_{idx}.txt".format(idx=idx)),"w", encoding="utf-8") as f:
            f.write(data[5])
        #end with

        '''this script just for limiting the number of output'''
        #if idx==5000: break
    #end for

    print("Making up corpus")
    corpus = preprocess.load_corpus(corpus_path)

    print("Making up dictionary")
    docs = preprocess.corpus2docs(corpus)
    dictionary = gensim.corpora.Dictionary(docs)
    dictionary.save(dictionary_full_path)

    print("Making up vector")
    vecs = preprocess.docs2vecs(docs,dictionary)
    gensim.corpora.MmCorpus.serialize(mmcorpus_full_path, vecs)
#end def

def get_dictionary():
    if not os.path.exists(dictionary_full_path): process()
    return gensim.corpora.Dictionary.load(dictionary_full_path)
#end def

def get_vector():
    if not os.path.exists(mmcorpus_full_path): process()
    return gensim.corpora.MmCorpus(mmcorpus_full_path)
#end def